@@ -51,11 +51,6 @@
 
 #define _(str) gettext (str)
 
-/* The results of open() in this file are not used with fchdir,
-   therefore save some unnecessary work in fchdir.c.  */
-#undef open
-#undef close
-
 
 /* Survey of Java compilers.
 
