@@ -771,14 +771,24 @@ check_header_entry (const message_ty *mp, const char *msgstr_string)
     ""
   };
   const size_t nfields = SIZEOF (required_fields);
+  /* FIXME: We could check if a required header field is missing and
+     report it as error.  However, it's could be too rigorous and
+     break backward compatibility.  */
+#if 0
   const size_t nrequiredfields = nfields - 1;
+#endif
   int seen_errors = 0;
   int cnt;
 
   for (cnt = 0; cnt < nfields; ++cnt)
     {
+#if 0
       int severity =
         (cnt < nrequiredfields ? PO_SEVERITY_ERROR : PO_SEVERITY_WARNING);
+#else
+      int severity =
+        PO_SEVERITY_WARNING;
+#endif
       const char *field = required_fields[cnt];
       size_t len = strlen (field);
       const char *line;
@@ -805,7 +815,8 @@ check_header_entry (const message_ty *mp, const char *msgstr_string)
                                    field);
                       po_xerror (severity, mp, NULL, 0, 0, true, msg);
                       free (msg);
-                      seen_errors++;
+                      if (severity == PO_SEVERITY_ERROR)
+                        seen_errors++;
                     }
                 }
               break;
@@ -821,7 +832,8 @@ check_header_entry (const message_ty *mp, const char *msgstr_string)
                        field);
           po_xerror (severity, mp, NULL, 0, 0, true, msg);
           free (msg);
-          seen_errors++;
+          if (severity == PO_SEVERITY_ERROR)
+            seen_errors++;
         }
     }
   return seen_errors;
