@@ -342,13 +342,14 @@ enum token_type_ty
   token_type_rparen,                    /* ) */
   token_type_lbrace,                    /* { */
   token_type_rbrace,                    /* } */
-  token_type_assign,                    /* = */
+  token_type_assign,                    /* = += -= *= /= %= <<= >>= &= |= ^= */
   token_type_return,                    /* return */
   token_type_plus,                      /* + */
-  token_type_minus,                     /* - */
+  token_type_arithmetic_operator,       /* - * / % << >> & | ^ */
   token_type_equality_test_operator,    /* == < > >= <= != */
   token_type_logic_operator,            /* ! && || */
   token_type_comma,                     /* , */
+  token_type_question,                  /* ? */
   token_type_colon,                     /* : */
   token_type_number,                    /* 2.7 */
   token_type_string_literal,            /* "abc" */
@@ -430,6 +431,19 @@ phase3_get (token_ty *tp)
   int bufpos;
   int last_was_backslash;
 
+#undef APPEND
+#define APPEND(c)                               \
+  do                                            \
+    {                                           \
+      if (bufpos >= bufmax)                     \
+        {                                       \
+          bufmax = 2 * bufmax + 10;             \
+          buffer = xrealloc (buffer, bufmax);   \
+        }                                       \
+      buffer[bufpos++] = c;                     \
+    }                                           \
+  while (0)
+
   if (phase3_pushback_length)
     {
       *tp = phase3_pushback[--phase3_pushback_length];
@@ -483,12 +497,7 @@ phase3_get (token_ty *tp)
           bufpos = 0;
           for (;;)
             {
-              if (bufpos >= bufmax)
-                {
-                  bufmax = 2 * bufmax + 10;
-                  buffer = xrealloc (buffer, bufmax);
-                }
-              buffer[bufpos++] = c;
+              APPEND (c);
               c = phase2_getc ();
               switch (c)
                 {
@@ -513,12 +522,7 @@ phase3_get (token_ty *tp)
                 }
               break;
             }
-          if (bufpos >= bufmax)
-            {
-              bufmax = 2 * bufmax + 10;
-              buffer = xrealloc (buffer, bufmax);
-            }
-          buffer[bufpos] = 0;
+          APPEND (0);
           if (strcmp (buffer, "return") == 0)
             tp->type = last_token_type = token_type_return;
           else
@@ -553,23 +557,13 @@ phase3_get (token_ty *tp)
           bufpos = 0;
           for (;;)
             {
-              if (bufpos >= bufmax)
-                {
-                  bufmax = 2 * bufmax + 10;
-                  buffer = xrealloc (buffer, bufmax);
-                }
-              buffer[bufpos++] = c;
+              APPEND (c);
               c = phase2_getc ();
               switch (c)
                 {
                 case 'e':
                 case 'E':
-                  if (bufpos >= bufmax)
-                    {
-                      bufmax = 2 * bufmax + 10;
-                      buffer = xrealloc (buffer, bufmax);
-                    }
-                  buffer[bufpos++] = c;
+                  APPEND (c);
                   c = phase2_getc ();
                   if (c != '+' && c != '-')
                     {
@@ -599,12 +593,7 @@ phase3_get (token_ty *tp)
                 }
               break;
             }
-          if (bufpos >= bufmax)
-            {
-              bufmax = 2 * bufmax + 10;
-              buffer = xrealloc (buffer, bufmax);
-            }
-          buffer[bufpos] = 0;
+          APPEND (0);
           tp->type = last_token_type = token_type_number;
           return;
 
@@ -622,6 +611,8 @@ phase3_get (token_ty *tp)
                 {
                 case '\\':
                   last_was_backslash = true;
+                  /* FALLTHROUGH */
+                default:
                   continue;
                 case '\n':
                   error_with_progname = false;
@@ -690,28 +681,24 @@ phase3_get (token_ty *tp)
                 bufpos = 0;
                 for (;;)
                   {
-                    c = phase2_getc ();
+                    /* Use phase 1, because phase 2 elides comments.  */
+                    c = phase1_getc ();
                     if (c == EOF)
                       break;
 
                     if (c == '"')
                       {
-                        int c2 = phase2_getc ();
+                        int c2 = phase1_getc ();
                         if (c2 == '"')
                           {
-                            int c3 = phase2_getc ();
+                            int c3 = phase1_getc ();
                             if (c3 == '"')
                               break;
-                            phase2_ungetc (c3);
+                            phase1_ungetc (c3);
                           }
-                        phase2_ungetc (c2);
+                        phase1_ungetc (c2);
                       }
-                    if (bufpos >= bufmax)
-                      {
-                        bufmax = 2 * bufmax + 10;
-                        buffer = xrealloc (buffer, bufmax);
-                      }
-                    buffer[bufpos++] = c;
+                    APPEND (c);
                   }
               }
             else
@@ -720,16 +707,11 @@ phase3_get (token_ty *tp)
                 bufpos = 0;
                 for (;;)
                   {
-                    c = phase2_getc ();
+                    c = phase1_getc ();
                     if (last_was_backslash)
                       {
                         last_was_backslash = false;
-                        if (bufpos >= bufmax)
-                          {
-                            bufmax = 2 * bufmax + 10;
-                            buffer = xrealloc (buffer, bufmax);
-                          }
-                        buffer[bufpos++] = c;
+                        APPEND (c);
                         continue;
                       }
 
@@ -739,12 +721,7 @@ phase3_get (token_ty *tp)
                         last_was_backslash = true;
                         /* FALLTHROUGH */
                       default:
-                        if (bufpos >= bufmax)
-                          {
-                            bufmax = 2 * bufmax + 10;
-                            buffer = xrealloc (buffer, bufmax);
-                          }
-                        buffer[bufpos++] = c;
+                        APPEND (c);
                         continue;
 
                       case '\n':
@@ -753,7 +730,7 @@ phase3_get (token_ty *tp)
 %s:%d: warning: unterminated string literal"),
                                logical_file_name, line_number - 1);
                         error_with_progname = true;
-                        phase2_ungetc ('\n');
+                        phase1_ungetc ('\n');
                         break;
                       case EOF: case '"':
                         break;
@@ -761,7 +738,7 @@ phase3_get (token_ty *tp)
                     break;
                   }
               }
-            buffer[bufpos] = 0;
+            APPEND (0);
             tp->type = last_token_type = template
               ? token_type_string_template : token_type_string_literal;
             tp->string = xstrdup (buffer);
@@ -778,10 +755,12 @@ phase3_get (token_ty *tp)
             case token_type_assign:
             case token_type_return:
             case token_type_plus:
-            case token_type_minus:
+            case token_type_arithmetic_operator:
             case token_type_equality_test_operator:
             case token_type_logic_operator:
             case token_type_comma:
+            case token_type_question:
+            case token_type_colon:
               phase3_scan_regex ();
               tp->type = last_token_type = token_type_regex_literal;
               break;
@@ -789,11 +768,12 @@ phase3_get (token_ty *tp)
               {
                 int c2 = phase2_getc ();
                 if (c2 == '=')
+                  tp->type = last_token_type = token_type_assign;
+                else
                   {
-                    /* /= */
                     phase2_ungetc (c2);
+                    tp->type = last_token_type = token_type_arithmetic_operator;
                   }
-                tp->type = last_token_type = token_type_other;
                 break;
               }
             }
@@ -820,9 +800,12 @@ phase3_get (token_ty *tp)
             int c2 = phase2_getc ();
             switch (c2)
               {
-              case '=': case '+':
+              case '+':
                 tp->type = last_token_type = token_type_other;
                 break;
+              case '=':
+                tp->type = last_token_type = token_type_assign;
+                break;
               default:
                 phase2_ungetc (c2);
                 tp->type = last_token_type = token_type_plus;
@@ -836,17 +819,34 @@ phase3_get (token_ty *tp)
             int c2 = phase2_getc ();
             switch (c2)
               {
-              case '=': case '-':
+              case '-':
                 tp->type = last_token_type = token_type_other;
                 break;
+              case '=':
+                tp->type = last_token_type = token_type_assign;
+                break;
               default:
                 phase2_ungetc (c2);
-                tp->type = last_token_type = token_type_minus;
+                tp->type = last_token_type = token_type_arithmetic_operator;
                 break;
               }
             return;
           }
 
+        case '%':
+        case '^':
+          {
+            int c2 = phase2_getc ();
+            if (c2 == '=')
+	      tp->type = last_token_type = token_type_assign;
+            else
+              {
+                phase2_ungetc (c2);
+                tp->type = last_token_type = token_type_logic_operator;
+              }
+            return;
+          }
+
         case '=':
           {
             int c2 = phase2_getc ();
@@ -870,12 +870,12 @@ phase3_get (token_ty *tp)
           {
             int c2 = phase2_getc ();
             if (c2 == '=')
+              tp->type = last_token_type = token_type_equality_test_operator;
+            else
               {
-                tp->type = last_token_type = token_type_equality_test_operator;
-                return;
+                phase2_ungetc (c2);
+                tp->type = last_token_type = token_type_logic_operator;
               }
-            phase2_ungetc (c2);
-            tp->type = last_token_type = token_type_logic_operator;
             return;
           }
           
@@ -888,17 +888,22 @@ phase3_get (token_ty *tp)
             else if (c2 == c)
               {
                 int c3 = phase2_getc ();
-                if (c3 != '=')
-                  phase2_ungetc (c3);
-                tp->type = last_token_type = token_type_other;
+                if (c3 == '=')
+                  tp->type = last_token_type = token_type_assign;
+                else
+                  {
+                    phase2_ungetc (c2);
+                    phase2_ungetc (c3);
+                    tp->type = last_token_type = token_type_other;
+                  }
               }
             else
               {
                 phase2_ungetc (c2);
                 tp->type = last_token_type = token_type_equality_test_operator;
               }
+            return;
           }
-          return;
           
         case ',':
           tp->type = last_token_type = token_type_comma;
@@ -915,25 +920,25 @@ phase3_get (token_ty *tp)
             if (c2 == c)
 	      tp->type = last_token_type = token_type_logic_operator;
             else if (c2 == '=')
-	      tp->type = last_token_type = token_type_other;
+	      tp->type = last_token_type = token_type_assign;
             else
               {
                 phase2_ungetc (c2);
-                tp->type = last_token_type = token_type_other;
+                tp->type = last_token_type = token_type_arithmetic_operator;
               }
+            return;
           }
-          return;
 
         case '?':
           {
             int c2 = phase2_getc ();
             if (c2 == '?')
+              tp->type = last_token_type = token_type_logic_operator;
+            else
               {
-                tp->type = last_token_type = token_type_logic_operator;
-                return;
+                phase2_ungetc (c2);
+                tp->type = last_token_type = token_type_question;
               }
-            phase2_ungetc (c2);
-            tp->type = last_token_type = token_type_other;
             return;
           }
 
@@ -942,6 +947,7 @@ phase3_get (token_ty *tp)
           return;
         }
     }
+#undef APPEND
 }
 
 static void
@@ -1191,9 +1197,10 @@ extract_balanced (message_list_ty *mlp, token_type_ty delim,
         case token_type_assign:
         case token_type_return:
         case token_type_plus:
-        case token_type_minus:
+        case token_type_arithmetic_operator:
         case token_type_equality_test_operator:
         case token_type_logic_operator:
+        case token_type_question:
         case token_type_colon:
         case token_type_number:
         case token_type_string_template:
